"""dj42 URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/2.0/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""


from django.contrib.auth import views as auth_views
from django.urls import path, re_path
from django.contrib import admin

from pwdtk.testviews import home
from pwdtk.testviews import logout_view
from pwdtk.testviews import protected

urlpatterns = [
    path('', home, name='pwdtk_test_home'),
    re_path(r'^login', auth_views.LoginView.as_view(),
            name='pwdtk_test_login'),
    re_path(r'^accounts/login', auth_views.LoginView.as_view(),
            name='pwdtk_test_login2'),
    re_path(r'^logout', logout_view,
            name='pwdtk_test_logout'),
    re_path(r'^protected', protected,
            name='pwdtk_test_protected'),
    re_path(r'^admin/', admin.site.urls),
    path('accounts/password_change/',
         auth_views.PasswordChangeView.as_view(),
         name='password_change'),
    path('accounts/password_change/done/',
         auth_views.PasswordChangeDoneView.as_view(),
         name='password_change_done'),
]



import json
import logging


from django.core.exceptions import MiddlewareNotUsed
from django.http import HttpResponse

try:
    from django.utils.deprecation import MiddlewareMixin
except: # noqa E722
    class MiddlewareMixin(object):
        pass


from pwdtk.auth_backends import PwdtkForceRenewException
from pwdtk.auth_backends import PwdtkLockedException
from pwdtk.helpers import PwdtkSettings


logger = logging.getLogger(__name__)


class PwdtkMiddleware(MiddlewareMixin):

    def __init__(self, get_response=None):

        if not PwdtkSettings.PWDTK_ENABLED:
            logger.debug("PWDTK middleware is disabled")
            raise MiddlewareNotUsed("pwdtk is disabled")
        if get_response:
            super(PwdtkMiddleware, self).__init__(
                get_response=get_response)
        else:
            super(PwdtkMiddleware, self).__init__()

    def process_request(self, request):
        if hasattr(PwdtkSettings, "reset_cache"):
            PwdtkSettings.reset_cache()

    def process_exception(self, request, exception):

        if isinstance(exception, PwdtkLockedException):
            context = exception.pwdtk_data.get_lockout_context()
            context["status"] = "PWDTK_LOCKED"
            return HttpResponse(
                json.dumps(context),
                content_type='application/json',
                status=403,
                )

        if isinstance(exception, PwdtkForceRenewException):
            return HttpResponse(
                json.dumps({"status": "PWDTK_NEED_RENEW_PASSWORD"}),
                content_type='application/json',
                status=403,
                )

        return None



# Generated by Django 2.2.28 on 2023-09-05 08:35

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('pwdtk', '0007_auto_20200304_1139'),
    ]

    operations = [
        migrations.AddField(
            model_name='pwddata',
            name='fake_username',
            field=models.CharField(max_length=150, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='pwddata',
            name='user',
            field=models.OneToOneField(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name='pwdtk_data',
                to=settings.AUTH_USER_MODEL),
        ),
    ]



from __future__ import absolute_import

import django
from django.conf import settings
from django.db import models
from django.utils import timezone

from pwdtk.helpers import PwdtkSettings

AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


def json_field(**kwargs):
    """
    Backward-compatible JSONField
    """
    if django.VERSION < (4, ):
        from django_jsonfield_backport.models import JSONField
        return JSONField(**kwargs)
    else:
        from django.db.models import JSONField
        return JSONField(**kwargs)


class PwdData(models.Model):
    """ a model in case no custom way is specified for storing related data
    """
    user = models.OneToOneField(AUTH_USER_MODEL,
                                related_name='pwdtk_data',
                                on_delete=models.CASCADE, null=True)
    fake_username = models.CharField(max_length=150, unique=True, null=True)
    locked = models.BooleanField(default=False)
    failed_logins = models.PositiveIntegerField(default=0)
    fail_time = models.DateTimeField(null=True)
    must_renew = models.BooleanField(default=False)
    last_change_time = models.DateTimeField(default=timezone.now)
    password_history = json_field(default=[])

    @classmethod
    def get_or_create_for_user(cls, user):
        if not hasattr(user, 'pwdtk_data'):
            user.pwdtk_data = cls.objects.create(user=user)
            cls.objects.filter(fake_username=user.username).delete()

        return user.pwdtk_data

    def __str__(self):
        return ("%r, %r" % ((self.user.id, self.user.username) if self.user else ("-", self.fake_username)))

    def set_locked(self, failed_logins=None):
        if failed_logins is not None:
            self.failed_logins = failed_logins
        self.locked = True
        self.fail_time = timezone.now()
        self.save()

    def is_locked(self):
        """ determines whether a user is still locked out
        """
        if not self.locked:
            return False

        if self.fail_age < PwdtkSettings.PWDTK_LOCKOUT_TIME:
            return True

        self.locked = False
        self.save()
        return False

    @property
    def aware_fail_time(self):
        if timezone.is_aware(self.fail_time):
            return self.fail_time
        else:
            if django.VERSION < (4, 0):
                import pytz
                return pytz.timezone(
                    PwdtkSettings.TIME_ZONE).localize(self.fail_time)
            else:
                from zoneinfo import ZoneInfo
                return self.fail_time.replace(
                    tzinfo=ZoneInfo(PwdtkSettings.TIME_ZONE))

    @property
    def fail_age(self):

        return int((timezone.now() - self.fail_time).total_seconds())

    def compute_must_renew(self):
        """ determines whether a user must renew his password
        """
        if getattr(self.user, "disable_must_renew", False):
            return False
        if PwdtkSettings.PWDTK_PASSWD_AGE == 0:
            return False
        return ((timezone.now() - self.last_change_time).total_seconds() >
                PwdtkSettings.PWDTK_PASSWD_AGE)

    def get_lockout_context(self):
        return {
            "username": self.user.username if self.user else self.fake_username,
            "lockout_time": PwdtkSettings.PWDTK_LOCKOUT_TIME,
            "failed_logins": self.failed_logins,
            "fail_time": (
                        self.aware_fail_time.isoformat()
                        if self.fail_time else None
                    ),
        }



from __future__ import absolute_import
from __future__ import print_function

import django
print(django.VERSION)
if django.VERSION < (2, ):
    from pwdtk.testproject.dj1.settings import *  # noqa: F401, F403
elif django.VERSION < (2, 2):
    from pwdtk.testproject.dj21.settings import *  # noqa: F401, F403
elif django.VERSION < (3, 1):
    from pwdtk.testproject.dj22.settings import *  # noqa: F401, F403
elif django.VERSION < (4, 2):
    from pwdtk.testproject.dj31.settings import *  # noqa: F401, F403
else:
    from pwdtk.testproject.dj42.settings import *  # noqa: F401, F403
